package data_manage

import (
	"encoding/json"
	"fmt"
	"hongze/hongze_ETA_mobile_api/controllers"
	"hongze/hongze_ETA_mobile_api/models"
	"hongze/hongze_ETA_mobile_api/models/data_manage"
	"hongze/hongze_ETA_mobile_api/models/system"
	"hongze/hongze_ETA_mobile_api/services/data"
	"hongze/hongze_ETA_mobile_api/utils"
	"time"
)

// ChartClassifyController 数据管理-分类模块
type ChartClassifyController struct {
	controllers.BaseAuthController
}

// ChartClassifyListV2
// @Title 图表分类列表
// @Description 图表分类列表接口
// @Param   IsShowMe   query   bool  true       "是否只看我的，true、false"
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /chart_classify/list [get]
func (this *ChartClassifyController) ChartClassifyListV2() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	resp := new(data_manage.ChartClassifyListResp)

	// 获取当前账号的不可见指标
	noPermissionChartIdMap := make(map[int]bool)
	{
		obj := data_manage.EdbInfoNoPermissionAdmin{}
		confList, err := obj.GetAllChartListByAdminId(this.SysUser.AdminId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "获取失败"
			br.ErrMsg = "获取不可见指标配置数据失败，Err:" + err.Error()
			return
		}
		for _, v := range confList {
			noPermissionChartIdMap[v.ChartInfoId] = true
		}
	}

	isShowMe, _ := this.GetBool("IsShowMe")
	if isShowMe {
		errMsg, err := getChartClassifyListForMe(*this.SysUser, resp)
		if err != nil {
			br.Msg = errMsg
			br.ErrMsg = err.Error()
			return
		}
		// 移除没有权限的图表
		allNodes := handleNoPermissionChart(resp.AllNodes, noPermissionChartIdMap)
		resp.AllNodes = allNodes

		br.Ret = 200
		br.Success = true
		br.Msg = "获取成功"
		br.Data = resp
		fmt.Println("source my classify")
		return
	}

	//判断是否存在缓存，如果存在缓存，那么直接从缓存中获取
	key := utils.CACHE_CHART_CLASSIFY
	if utils.Re == nil {
		if utils.Re == nil && utils.Rc.IsExist(key) {
			if data, err1 := utils.Rc.RedisBytes(key); err1 == nil {
				err := json.Unmarshal(data, &resp)
				if err == nil && resp != nil {
					// 移除没有权限的图表
					allNodes := handleNoPermissionChart(resp.AllNodes, noPermissionChartIdMap)
					resp.AllNodes = allNodes

					br.Ret = 200
					br.Success = true
					br.Msg = "获取成功"
					br.Data = resp
					fmt.Println("source redis")
					return
				}
			}
		}
	}

	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_DEFAULT)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	classifyAll, err := data_manage.GetChartClassifyAll(utils.CHART_SOURCE_DEFAULT)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	allChartInfo, err := data_manage.GetChartInfoAll(utils.CHART_SOURCE_DEFAULT)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取图表信息失败，Err:" + err.Error()
		return
	}

	chartInfoMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range allChartInfo {
		chartInfoMap[v.ChartClassifyId] = append(chartInfoMap[v.ChartClassifyId], v)
	}
	rootChildMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range classifyAll {
		rootChildMap[v.ParentId] = append(rootChildMap[v.ParentId], v)
		if existItems, ok := chartInfoMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
	}
	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	for _, v := range rootList {
		if existItems, ok := rootChildMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
		nodeAll = append(nodeAll, v)
	}
	//
	//
	//nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	//for k := range rootList {
	//	rootNode := rootList[k]
	//	data.ChartClassifyListMakeTree(classifyAll, rootNode)
	//	nodeAll = append(nodeAll, rootNode)
	//}
	resp.AllNodes = nodeAll

	// 将数据加入缓存
	if utils.Re == nil {
		data, _ := json.Marshal(resp)
		utils.Rc.Put(key, data, 2*time.Hour)
	}

	// 移除没有权限的图表
	allNodes := handleNoPermissionChart(resp.AllNodes, noPermissionChartIdMap)
	resp.AllNodes = allNodes

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// getChartClassifyListForMe 获取我创建的图表
func getChartClassifyListForMe(adminInfo system.Admin, resp *data_manage.ChartClassifyListResp) (errMsg string, err error) {
	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_DEFAULT)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	classifyAll, err := data_manage.GetChartClassifyAll(utils.CHART_SOURCE_DEFAULT)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	allChartInfo, err := data_manage.GetChartInfoByAdminId(utils.CHART_SOURCE_DEFAULT, adminInfo.AdminId)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	chartInfoMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range allChartInfo {
		chartInfoMap[v.ChartClassifyId] = append(chartInfoMap[v.ChartClassifyId], v)
	}
	rootChildMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range classifyAll {
		rootChildMap[v.ParentId] = append(rootChildMap[v.ParentId], v)
		if existItems, ok := chartInfoMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
	}
	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	for _, v := range rootList {
		if existItems, ok := rootChildMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
		nodeAll = append(nodeAll, v)
	}
	resp.AllNodes = nodeAll

	return
}

// handleNoPermissionChart 图表列表返回，将没有权限的图表移除
func handleNoPermissionChart(allNodes []*data_manage.ChartClassifyItems, noPermissionChartIdMap map[int]bool) (newAllNodes []*data_manage.ChartClassifyItems) {
	// 移除没有权限的图表
	newAllNodes = make([]*data_manage.ChartClassifyItems, 0)
	for _, node := range allNodes {
		// 二级分类
		tmpNodeInfo := *node
		tmpNodeList := make([]*data_manage.ChartClassifyItems, 0)
		if node.Children != nil {
			for _, chartList := range node.Children {
				tmpInfo := *chartList
				tmpList := make([]*data_manage.ChartClassifyItems, 0)

				if chartList.Children != nil {
					for _, chartInfo := range chartList.Children {
						// 如果指标不可见，那么就不返回该指标
						if _, ok := noPermissionChartIdMap[chartInfo.ChartInfoId]; ok {
							continue
						}
						tmpList = append(tmpList, chartInfo)
					}
				}
				tmpInfo.Children = tmpList
				tmpNodeList = append(tmpNodeList, &tmpInfo)
			}
		}
		tmpNodeInfo.Children = tmpNodeList
		newAllNodes = append(newAllNodes, &tmpNodeInfo)
	}

	return
}

// ChartClassifyItems
// @Title 获取所有图表分类接口-不包含图表
// @Description 获取所有图表分类接口-不包含图表
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /chart_classify/items [get]
func (this *ChartClassifyController) ChartClassifyItems() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_DEFAULT)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	classifyAll, err := data_manage.GetChartClassifyAll(utils.CHART_SOURCE_DEFAULT)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	for k := range rootList {
		rootNode := rootList[k]
		data.ChartClassifyItemsMakeTree(classifyAll, rootNode)
		nodeAll = append(nodeAll, rootNode)
	}

	language := `CN`
	// 指标显示的语言
	{
		configDetail, _ := system.GetConfigDetailByCode(this.SysUser.AdminId, system.ChartLanguageVar)
		if configDetail != nil {
			language = configDetail.ConfigValue
		} else {
			configDetail, _ = system.GetDefaultConfigDetailByCode(system.ChartLanguageVar)
			if configDetail != nil {
				language = configDetail.ConfigValue
			}
		}
	}

	resp := new(data_manage.ChartClassifyListResp)
	resp.AllNodes = nodeAll
	resp.Language = language
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// DeleteChartClassify
// @Title 删除图表分类/图表
// @Description 删除图表分类/图表接口
// @Param	request	body data_manage.DeleteChartClassifyReq true "type json string"
// @Success 200 Ret=200 删除成功
// @router /chart_classify/delete [post]
func (this *ChartClassifyController) DeleteChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_manage.DeleteChartClassifyReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ChartClassifyId < 0 && req.ChartInfoId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	//删除分类
	if req.ChartClassifyId > 0 && req.ChartInfoId == 0 {
		//判断是否含有指标
		count, err := data_manage.GetChartInfoCountByClassifyId(req.ChartClassifyId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "判断名称是否已存在失败，Err:" + err.Error()
			return
		}

		if count > 0 {
			br.Msg = "该目录下存在关联指标，不可删除"
			br.IsSendEmail = false
			return
		}

		err = data_manage.DeleteChartClassify(req.ChartClassifyId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败,Err:" + err.Error()
			return
		}
	}
	resp := new(data_manage.AddChartInfoResp)
	//删除图表
	if req.ChartInfoId > 0 {
		chartInfo, err := data_manage.GetChartInfoById(req.ChartInfoId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				br.Msg = "图表已删除，请刷新页面"
				br.ErrMsg = "指标不存在,Err:" + err.Error()
				return
			} else {
				br.Msg = "删除失败"
				br.ErrMsg = "删除失败，获取指标信息失败，Err:" + err.Error()
				return
			}
		}
		if chartInfo == nil {
			br.Msg = "图表已删除，请刷新页面"
			return
		}
		//图表操作权限
		ok := data.CheckOpChartPermission(sysUser, chartInfo.SysUserId)
		if !ok {
			br.Msg = "没有该图表的操作权限"
			br.ErrMsg = "没有该图表的操作权限"
			return
		}

		// 获取引用该图表的MyCharts, 用于ES删除
		var myCond string
		var myPars []interface{}
		myCond += ` AND a.chart_info_id = ? `
		myPars = append(myPars, req.ChartInfoId)
		myCharts, e := data_manage.GetMyChartListGroupByCharyInfoIdAndAdminIdByCondition(myCond, myPars)
		if e != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "获取引用图表的MyChats失败, Err: " + e.Error()
			return
		}
		myIds := make([]int, 0)
		for _, m := range myCharts {
			myIds = append(myIds, m.MyChartId)
		}

		//删除图表及关联指标
		err = data_manage.DeleteChartInfoAndData(req.ChartInfoId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败，Err:" + err.Error()
			return
		}

		//删除ES
		{
			go data.EsDeleteChartInfo(req.ChartInfoId)
			// 删除MY ETA 图表 es数据
			//go data.EsDeleteMyChartInfoByChartInfoId(req.ChartInfoId)
			go data.EsDeleteMyChartInfoByMyChartIds(myIds)
		}

		var condition string
		var pars []interface{}
		condition += " AND chart_classify_id=? AND source = ?  "
		pars = append(pars, chartInfo.ChartClassifyId, utils.CHART_SOURCE_DEFAULT)

		condition += " AND chart_info_id>? ORDER BY create_time ASC LIMIT 1 "
		pars = append(pars, req.ChartInfoId)

		nextItem, err := data_manage.GetChartInfoByCondition(condition, pars)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
			return
		}

		if nextItem != nil {
			resp.UniqueCode = nextItem.UniqueCode
			resp.ChartInfoId = nextItem.ChartInfoId
		} else {
			var condition string
			var pars []interface{}

			condition += " AND level=3 "
			//pars = append(pars, chartInfo.ChartClassifyId)

			condition += " AND chart_classify_id>? ORDER BY chart_classify_id ASC LIMIT 1 "
			pars = append(pars, chartInfo.ChartClassifyId)

			classifyItem, err := data_manage.GetChartClassifyByCondition(condition, pars)
			if err != nil && err.Error() != utils.ErrNoRow() {
				br.Msg = "删除失败"
				br.ErrMsg = "获取下一级图库分类信息失败，Err:" + err.Error()
				return
			}
			if classifyItem != nil {
				nextItem, err = data_manage.GetNextChartInfo(chartInfo.ChartClassifyId)
				if err != nil && err.Error() != utils.ErrNoRow() {
					br.Msg = "删除失败"
					br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
					return
				}
				if nextItem != nil {
					resp.UniqueCode = nextItem.UniqueCode
					resp.ChartInfoId = nextItem.ChartInfoId
				}
			}
		}
		//新增操作日志
		{
			chartLog := new(data_manage.ChartInfoLog)
			chartLog.ChartName = chartInfo.ChartName
			chartLog.ChartInfoId = req.ChartInfoId
			chartLog.ChartClassifyId = chartInfo.ChartClassifyId
			chartLog.SysUserId = sysUser.AdminId
			chartLog.SysUserRealName = sysUser.RealName
			chartLog.UniqueCode = chartInfo.UniqueCode
			chartLog.CreateTime = time.Now()
			chartLog.Content = string(this.Ctx.Input.RequestBody)
			chartLog.Status = "删除图表"
			chartLog.Method = this.Ctx.Input.URI()
			go data_manage.AddChartInfoLog(chartLog)
		}
	}
	br.Ret = 200
	br.Msg = "删除成功"
	br.Success = true
	br.Data = resp
	br.IsAddLog = true
}
